#!/bin/bash

function about(){
 echo "$(basename ${BASH_SOURCE[0]}) version 20240731";
 echo "##############################################################";
 echo "# Shell Installation Script.                                 #";
 echo "# Copyright (c) 2023-2024 Alexey Kuryakin daqgroup@mail.ru   #";
 echo "##############################################################";
};

function note(){ return; };
function fatal(){ echo -ne "\n$1\n\n"; exit 1; };

note "Script identification.";
note "######################";
readonly startupdir="$(pwd -LP)";
readonly scriptfile="${BASH_SOURCE[0]}";
readonly scriptname="$(basename $scriptfile)";
readonly scriptbase="$(basename $scriptfile .sh)";
readonly scripthome="$(dirname  $scriptfile)";
readonly scriptFILE="$(realpath $scriptfile)";
readonly scriptHOME="$(dirname  $scriptFILE)";

note "Extract target package name from script name.";
note "#############################################";
readonly targetname="$(echo "$scriptbase" | sed 's/^[a-z]*-//')";

note "The payload directory contains user installer + data archive(s).";
note "################################################################";
readonly payloaddir="$scriptHOME";

note "Include the install-library.sh library source.";
note "##############################################";
readonly installlib="$payloaddir/install-library.sh";
source $installlib || fatal "Error: not found $installlib.";

note "##############################################";
note "Try install silent .run or .deb package in arg";
note "##############################################";
function try_install_silent(){
 local inst="";
 local post="";
 local pkg="$1";
 local pkn="$pkg";
 pkn="$(basename $pkn .run)";
 pkn="$(basename $pkn .deb)";
 pkn="$(echo $pkn | sed 's/install-//')";
 echo;
 echo "Try install package $pkn from $pkg";
 echo;
 case $pkg in
  *.run) post="-- --silent"; ;;
  *.deb) inst="sudo dpkg -i"; if check_dpkg_installed "$pkn" && is_same_deb_version "$pkn" "$pkg"; then return 0; fi; ;;
  *) echo "Warning: invalid package - $pkg"; return 1; ;;
 esac;
 if $inst $pkg $post; then
  echo "Installed: package - $pkg";
  return 0;
 else
  echo "Warning: failed package - $pkg";
  return 1;
 fi;
};

note "#######################";
note "Install target package.";
note "#######################";
function install_target(){
 echo "";
 echo "Install: $targetname";
 echo "";
 common_predinst;
 note "##################################";
 note "### Begin User Installer code: ###";
 note "##################################";
 
 note "#########################################";
 note "Check the script is running in user mode.";
 note "#########################################";
 check_confirm_iam_user || fatal "Cancel: user mode expected.";
 note "########################################";
 note "Check required packages to be installed.";
 note "########################################";
 local packages="zenity makeself zip unzip xz-utils p7zip";
 packages="$packages doublecmd-gtk";
 check_install_dpkg $packages;
 note "#########################################";
 note "Select packages to install by zenity GUI.";
 note "#########################################";
 local ans="$(ask_zenity 0 \
  --title "$scriptname …" \
  --text "<big>$(langstr en "Select packages to install:" ru "Выбор пакетов для инсталляции:")</big>" \
  --list --checklist --separator " " --print-column 2 \
  --column "$(langstr en "Check" ru "Выбор")" --column " $(langstr en "Package" ru "Пакет")" --column "$(langstr en "Comment" ru "Комментарий")" \
  true  install-generic.run                     " $(langstr en "Generic *.run installer" ru "Шаблон *.run инсталлятора")" \
  true  install-kate-settings.run               " $(langstr en "Settings for editor kate" ru "Настройки для редактора kate")" \
  true  install-doublecmd-settings.run          " $(langstr en "Settings for DoubleCommander" ru "Настройки для DoubleCommander")" \
  true  install-daqgroup-markdown-viewer.run    " $(langstr en "Markdown file viewer" ru "Программе просмотра файлов Markdown")" \
  true  install-daqgroup-crwlib.run             " $(langstr en "Code library (Pascal) for CRW-DAQ package" ru "Библиотеки кодов (Pascal) пакета CRW-DAQ")" \
  false install-fpcupdeluxe-min.run             " $(langstr en "Free Pascal Compiler + IDE (MIN)" ru "Компилятор и среда разработки Free Pascal (MIN)")" \
  false install-fpcupdeluxe-max.run             " $(langstr en "Free Pascal Compiler + IDE (MAX)" ru "Компилятор и среда разработки Free Pascal (MAX)")" \
  true  install-zenity-common.deb               " $(langstr en "zenity-common: Libraries for Zenity" ru "zenity-common: Библиотеки для Zenity")" \
  true  install-zenity.deb                      " $(langstr en "zenity: Dialog system for Shell scripting" ru "zenity: Диалоговая система для сценариев Shell")" \
  true  install-wmctrl.deb                      " $(langstr en "wmctrl: Window control utility" ru "wmctrl: Утилита для управления окнами")" \
  true  install-gtkterm.deb                     " $(langstr en "gtkterm: COM port terminal with GUI" ru "gtkterm: Графический терминал COM порта")" \
  true  install-cutecom.deb                     " $(langstr en "cutecom: COM port terminal with GUI" ru "cutecom: Графический терминал COM порта")" \
  true  install-cr3.deb                         " $(langstr en "cr3: e-book reader CoolReader" ru "cr3: Чтение электронных книг CoolReader")" \
  true  install-glogg.deb                       " $(langstr en "glogg: LOG files viewer with GUI" ru "glogg: Графический обозреватель журналов (LOG файлов)")" \
  true  install-klogg.deb                       " $(langstr en "klogg: LOG files viewer with GUI" ru "klogg: Графический обозреватель журналов (LOG файлов)")" \
  false install-ser2net.deb                     " $(langstr en "ser2net: COM-to-TCP/IP tunnel server" ru "ser2net: Сервер для туннеля COM-в-TCP/IP")" \
  false install-virtualbox.deb                  " $(langstr en "virtualbox: Virtual Machine Hypervisor" ru "virtualbox: Гипервизор Виртуальных Машин")" \
  true  install-daqgroup-dim.deb                " $(langstr en "DIM: CERN network library" ru "DIM: Сетевая распределенная информационная система CERN")" \
  true  install-daqgroup-smi.deb                " $(langstr en "SMI: CERN State Manager Interface System" ru "SMI: Система сетевых Конечных Автоматов FSM от CERN")" \
  true  install-daqgroup-dunst.deb              " $(langstr en "DUNST: desktop notification system" ru "DUNST: мощная система всплывающих уведомлений")" \
  true  install-daqgroup-root.deb               " $(langstr en "ROOT: CERN data analisys framework" ru "ROOT: система анализа данных от CERN")" \
  true  install-daqgroup-diesel.deb             " $(langstr en "diesel: Pascal Interpreter + IDE" ru "diesel: Интерпретатор Дизель Паскаль и среда разработки")" \
  true  install-daqgroup-diesel-ext.run         " $(langstr en "diesel: Addon external libraries" ru "diesel: Стандартные библиотеки для языка Дизель Паскаль")" \
  true  install-daqgroup-rhvoice.deb            " $(langstr en "RHVoice: Multilanguage Speech Synthesizer" ru "RHVoice: многоязычный речевой синтезатор")" \
  true  install-daqgroup-crwlib.run             " $(langstr en "crwlib: Common libraries for CRW-DAQ" ru "crwlib: Общие библиотеки для пакета CRW-DAQ")" \
  true  install-daqgroup-crwkit.run             " $(langstr en "crwkit: Common utilities for CRW-DAQ" ru "crwkit: Общие утилиты нужные для пакета CRW-DAQ")" \
  true  install-daqgroup-crwdaq.run             " $(langstr en "crwdaq: The CRW-DAQ package (main files)" ru "crwdaq: Пакет CRW-DAQ (основные файлы)")" \
  --timeout 600 --width 860 --height 760 \
  2>/dev/null)";
 if [ -n "$ans" ]; then
  echo "Selected packages:"; local pki=0;
  for pkg in $ans; do let pki+=1; echo " $pki: $pkg"; done; echo;
 else
  fatal "Cancel: packages not selected.";
 fi;
 note "###############################";
 note "Try to install select packages.";
 note "###############################";
 local pkdir="$USER_PWD"; echo "USER_PWD=$USER_PWD";
 local pkdir="/srv/public/mirror/addons/daqgroupkit";
 if [ ! -e "$pkdir" ]; then pkdir="$USER_PWD"; fi;
 if pushd "$pkdir"; then
  local numinst=0;
  local numbugs=0;
  for pkg in $ans; do
   if [ -e "$pkg" ]; then
    if try_install_silent $pkdir/$pkg; then let numinst+=1; else let numbugs+=1; fi;
   else
    echo "Warning: not found - $pkg";
    let numbugs+=1;
   fi;
  done;
 fi;
 popd;
 note "#######################################";
 note "Notify summary results of installation.";
 note "#######################################";
 local summary="Packages installed: $numinst, failed: $numbugs.";
 if [ $numbugs -eq 0 ]; then
  notify_ok_apply "$scriptname" "$summary" 20;
 else
  notify_error    "$scriptname" "$summary" 60;
 fi;
 
 note "##################################";
 note "### End of User Installer code ###";
 note "##################################";
 common_postinst;
};

note "########";
note "# Main #";
note "########";
function main(){
 about; print_identification; sleep 1;
 check_sudo_is_avail || fatal "Error: sudo is not available.";
 check_install_package zenity makeself zip unzip xz:xz-utils 7za:p7zip;
 local ask="$(zen_hello_user)\n\n$(zen_do_you_want_to $(langstr en 'Run' ru 'Запустить'))\n";
 ask="$ask\n<big><b><span fgcolor='green'>install-$targetname.sh</span></b>?</big>";
 if ask_zenity 0 --question --text "$ask" --timeout 180 >/dev/null 2>&1; then
  install_target;
  echo -ne "\nDone\n\n";
 else
  echo -ne "\nCancel\n\n";
 fi;
};

mkdir -p $HOME/.local/share/makeself;

main "$@" | tee $HOME/.local/share/makeself/$scriptbase.log;

#############
# END OF FILE
#############
